from django.apps import AppConfig
from django.db import models
from django.db.models import ManyToOneRel, OneToOneRel, ManyToManyRel, ManyToManyField, UUIDField, FileField, \
    ForeignKey, OneToOneField, DateField, TimeField, DateTimeField, BigAutoField
from django.db.models.options import Options


class VModel(models.Model):
    objects = models.Manager()
    sort = models.BigIntegerField(null=True, verbose_name='序号')
    name = models.CharField(max_length=50, null=True)
    create_time = models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')
    modify_time = models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')
    saved = models.BooleanField(default=False)

    class Meta:
        abstract = True

    def __str__(self):
        return str(self.full())

    def get_meta(self)->Options:
        return getattr(self, '_meta')

    def get_entity(self):
        name = type(self).__name__
        config: AppConfig = self.get_meta().app_config
        return f'{config.label}.{name}'

    def json(self, *args, **kwargs):
        excludes = [ManyToOneRel, OneToOneRel, ManyToManyField, ManyToManyRel, UUIDField]
        fields = [field for field in self.get_meta().get_fields() if type(field) not in excludes]
        data = {}
        for field in fields:
            value = field.value_from_object(self)
            prop = field.name
            domain = type(field)
            if value is not None:
                if domain in [ForeignKey, OneToOneField]:
                    prop = f'{prop}_id'
                elif domain in [DateTimeField]:
                    value = value.strftime('%Y-%m-%d %H:%M:%S')
                elif domain in [DateField]:
                    value = value.strftime('%Y-%m-%d')
                elif domain in [TimeField]:
                    value = value.strftime('%H:%M:%S')
                elif domain in [FileField]:
                    value = value.name
                elif domain in [BigAutoField]:
                    value = str(value)
            data[prop] = value
        data.update({f'${k}': v for k,v in kwargs.items()})
        return data

    def full(self):
        includes = [ManyToManyField, ManyToManyRel, ForeignKey, ManyToOneRel, OneToOneField, OneToOneRel]
        fields = [field for field in self.get_meta().get_fields() if type(field) in includes]
        data = self.json()
        for field in fields:
            prop = field.name
            domain = type(field)
            if domain in [ForeignKey,OneToOneField]:
                bean: VModel = getattr(self, prop)
                data[prop] = bean.json() if bean else None
            elif domain == OneToOneRel:
                print('OneToOneRel')
                pass
            elif domain in [ManyToManyField,ManyToManyRel,ManyToOneRel]:
                accessor = prop if domain ==  ManyToManyField else field.get_accessor_name()
                _set = getattr(self, accessor).all().order_by('-sort')
                data[prop] = [item.id for item in _set]
                data[f'{prop}_set'] = [item.json() for item in _set]
        return data


class VTree(VModel):
    pid = models.IntegerField(null=False, default=0, verbose_name='父节点')
    isLeaf = models.BooleanField( default=False, verbose_name='叶子节点')
    icon = models.CharField(max_length=255, null=True, verbose_name='图标')
    class Meta:
        abstract = True




from django.db import models
from .core import VModel


class Meta(VModel):
    db = models.CharField(max_length=100, verbose_name='数据库', null=True)
    entity = models.CharField(max_length=100, verbose_name='数据源', null=True)
    name = models.CharField(max_length=50, verbose_name='实体别名', null=True)

    class Meta:
        verbose_name = '数据实体'
        unique_together = ('db', 'entity')


class MetaView(VModel):
    meta = models.ForeignKey('Meta', on_delete=models.CASCADE, verbose_name='元数据')

    code = models.CharField(max_length=50, verbose_name='类视图', default='default ')
    name = models.CharField(max_length=50, verbose_name='视图名称', null=True)

    form_width = models.IntegerField(default=0, verbose_name='表单宽度')
    form_height = models.IntegerField(default=0, verbose_name='表单高度')
    table_width = models.IntegerField(default=0, verbose_name='表格宽度')
    table_height = models.IntegerField(default=0, verbose_name='表格高度')

    enable = models.BooleanField(default=True, verbose_name='是否启用')
    show_header = models.BooleanField(default=True, verbose_name='展示头部')
    allow_search = models.BooleanField(default=True, verbose_name='检索功能')
    allow_sort = models.BooleanField(default=True, verbose_name='移动功能')
    allow_order = models.BooleanField(default=True, verbose_name='排序功能')
    allow_insert = models.BooleanField(default=True, verbose_name='新增功能')
    allow_edit = models.BooleanField(default=True, verbose_name='编辑功能')
    allow_remove = models.BooleanField(default=True, verbose_name='删除功能')
    allow_download = models.BooleanField(default=True, verbose_name='下载功能')
    allow_upload = models.BooleanField(default=True, verbose_name='上传功能')

    class Meta:
        verbose_name = '数据视图'
        unique_together = ('meta', 'code')


class MetaField(VModel):
    # 标识
    view = models.ForeignKey('MetaView', on_delete=models.CASCADE, verbose_name='数据视图')
    prop = models.CharField(max_length=100, verbose_name='字段名称')  #
    label = models.CharField(max_length=100, verbose_name='字段标签')  #
    name = models.CharField(max_length=100, verbose_name='字段别名')  #

    """tool"""
    domain = models.CharField(max_length=100, verbose_name='字段类型')  #
    tool = models.CharField(max_length=100, default='default', verbose_name='工具组件')
    refer = models.JSONField(default=dict, verbose_name='索引')  #
    format = models.JSONField(default=dict, verbose_name='格式')  #


    """rest"""
    not_null = models.BooleanField(default=False, verbose_name='不为空') #
    allow_edit = models.BooleanField(default=True, verbose_name='可编辑')
    allow_order = models.BooleanField(default=True, verbose_name='可排序')
    allow_search = models.BooleanField(default=True, verbose_name='可搜索')
    allow_download = models.BooleanField(default=True, verbose_name='可下载')
    allow_upload = models.BooleanField(default=False, verbose_name='可上传')
    allow_update = models.BooleanField(default=False, verbose_name='可更新')

    """table"""
    unit = models.CharField(max_length=55, verbose_name='单位符', null=True)
    column_width = models.FloatField(default=0, verbose_name='表头宽度')
    align = models.CharField(max_length=55, default='left', verbose_name='对齐方式') #
    fixed = models.CharField(max_length=100, verbose_name='固定位置', null=True)
    header_color = models.CharField(max_length=55, verbose_name='表头颜色', null=True)
    cell_color = models.CharField(max_length=55, verbose_name='单元颜色', null=True)
    edit_on_table = models.BooleanField(default=True, verbose_name='表格编辑')
    hide_on_table = models.BooleanField(default=False, verbose_name='表内隐藏')

    """form"""
    span = models.IntegerField(default=0, verbose_name='表单占位')
    hide_on_form = models.BooleanField(default=False, verbose_name='表单隐藏')
    hide_on_form_edit = models.BooleanField(default=False, verbose_name='编辑隐藏')
    hide_on_form_insert = models.BooleanField(default=False, verbose_name='新增隐藏')
    hide_on_form_branch = models.BooleanField(default=False, verbose_name='分支隐藏')
    hide_on_form_leaf = models.BooleanField(default=False, verbose_name='叶子隐藏')


    class Meta:
        verbose_name = '视图字段'





import asyncio

async def execute_channel(method, sender):
    thread = asyncio.to_thread(__execute__, method, sender)
    asyncio.create_task(thread)


def __execute__(method, sender):
    sender.to_clients(None, [sender.client], 'start')
    method(sender)
    sender.to_clients(None, [sender.client], 'stop')





from django.db import models
from .core import VModel, VTree


class MetaFieldTool(VTree):
    name = models.CharField(max_length=255, null=True, verbose_name='名称')
    code = models.CharField(max_length=100, unique=True, null=True, verbose_name='代码')  #

    class Meta:
        verbose_name = '元数据字段工具'
#
#

class MetaFieldDomain(VModel):
    name = models.CharField(max_length=255, unique=True, null=True, verbose_name='名称')
    tools = models.ManyToManyField(to=MetaFieldTool, verbose_name='工具集')
    default = models.ForeignKey(
        to=MetaFieldTool, null=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name='默认工具')
    search = models.ForeignKey(
        to=MetaFieldTool, null=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name='搜索工具')
    align = models.CharField(max_length=10, null=True, verbose_name='对齐方式')
    class Meta:
        verbose_name = '元数据字段类型'




from django.http import JsonResponse

class ValarResponse(JsonResponse):
    def __init__(self, data, message='', code='info'):
        self.message = message
        self.code = code
        super(ValarResponse, self).__init__(data, safe=False)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

requires = [
    'channels==3.0.3',
    'pymongo~=4.11.2',
    'asgiref~=3.8.1',
    'django-cors-headers==4.2.0',
    'pandas==2.2.3',
    'openpyxl==3.1.5',
    'deepmerge~=2.0',
    'minio==7.2.2'
]

setup(
    name="valar",  # 包名
    version="1.1.1",  # 版本号
    author="LYP",  # 作者
    author_email="liuyinpeng@buaa.edu.cn",  # 邮箱
    description="valar for morghulis",  # 简短描述
    long_description=long_description,  # 详细说明
    long_description_content_type="text/markdown",  # 详细说明使用标记类型
    url="https://gitee.com/GRIFFIN120/valar_dev",                     # 项目主页
    packages=find_packages(where="valar"),  # 需要打包的部分
    package_dir={"": "valar"},  # 设置src目录为根目录
    python_requires=">=3.9",  # 项目支持的Python版本
    install_requires=requires,                     # 项目必须的依赖
    include_package_data=False  # 是否包含非Python文件（如资源文件）
)


import importlib

from django.conf import settings
from django.urls import path, include

class AutoUrlPatternsMixin:
    name = None  # 子类必须提供
    def set_url(self):
        root = settings.ROOT_URLCONF
        module = importlib.import_module(root)
        urlpatterns: list = getattr(module, 'urlpatterns')
        url = f'{self.name}.urls'
        urlpatterns.append(path('data/', include(url)))


